# -*- coding: utf-8 -*-
"""

Analyzing convergence of IQIS ensemble statistics as
a function of number of members
v1

"""


import os
import numpy as np
import netCDF4 as nc
import matplotlib.pyplot as plt
from scipy import stats

nruns           = 500 #nb of ensemble members
nsteps          = 2000 #nb of time steps
nmembsplot      = np.arange(10,501,10)

DirResults   = os.path.dirname(os.path.realpath(__file__))+'/'

NameRes   = ['WhiteNoiseSMB',
             'WhiteNoiseCalving',
             'WhiteNoiseClvSMB',
             'ARTF']
NameLabels = [r'$IQIS_1$',
              r'$IQIS_2$',
              r'$IQIS_3$',
              r'$IQIS_4$']

ls_colors   = plt.cm.rainbow(np.linspace(0,1,len(NameRes)))

### Random selection of indices ###
isel = []
for nn,num in enumerate(nmembsplot):
    if(nn==0):
        isel.append(np.random.randint(0,nruns,size=num))
    else:
        allinds  = np.random.randint(0,nruns,nruns).tolist() #all indices in random order
        numextra = num-nmembsplot[nn-1]
        indsnew  = [ind for ind in allinds if((ind in isel[nn-1])==False)][0:numextra]
        isel.append(np.append(isel[nn-1],indsnew))
        
for ii,runtype in enumerate(NameRes):
    filename = [file for file in os.listdir(DirResults) if f'NCfileIQIS_{runtype}_' in file][0]
    ds = nc.Dataset(DirResults+filename)

    ### Load ###
    timear         = np.array(ds.variables['time'])
    if(ii==0):
        ### Prepare arrays ###
        MnFullMvals      = np.zeros((len(NameRes),len(nmembsplot),len(timear)))
        SdevFullMvals    = np.zeros((len(NameRes),len(nmembsplot),len(timear)))
        SkewFullMvals    = np.zeros((len(NameRes),len(nmembsplot),len(timear)))
        MnSelMvals       = np.zeros((len(NameRes),len(nmembsplot),len(timear)))
        SdevSelMvals     = np.zeros((len(NameRes),len(nmembsplot),len(timear)))
        SkewSelMvals     = np.zeros((len(NameRes),len(nmembsplot),len(timear)))

    fullmatMvals       = 917*1e-12*np.array(ds.variables['volume'])[0:nruns,:] #[Gt]
    ds.close()
    ### Stats on the full ensemble ###
    MnFullMvals[ii,:,:]         = np.mean(fullmatMvals,axis=0)
    SdevFullMvals[ii,:,:]       = np.std(fullmatMvals,axis=0)
    SkewFullMvals[ii,:,:]       = stats.skew(fullmatMvals,axis=0)
    ### Stats on the random selections ###
    for jj in range(len(nmembsplot)):
        selmatMvals             = fullmatMvals[isel[jj],:]
        MnSelMvals[ii,jj,:]     = np.mean(selmatMvals,axis=0)
        SdevSelMvals[ii,jj,:]   = np.std(selmatMvals,axis=0)
        SkewSelMvals[ii,jj,:]   = stats.skew(selmatMvals,axis=0)


### Take relative values to the full ensemble values ###
relMnSelMvals    = MnSelMvals/MnFullMvals
relSdevSelMvals  = SdevSelMvals/SdevFullMvals

### Plotting ###
fig = plt.figure(figsize=[11,10])
ax  = plt.subplot(311)
for rr,taulevel in enumerate(NameRes):
    ax.plot(nmembsplot,1e-3*MnSelMvals[rr,:,-1],c=ls_colors[rr],linewidth=1.0,marker='o',markersize=4,label=NameLabels[rr])
ax.tick_params(axis='y', which='major', labelsize=14)
ax.set_ylabel(r'Mean final ice mass [$\times 10^3$ Gt]',fontsize=14)
ax.tick_params(axis='y', which='major',labelsize=14)
ax.tick_params(axis='x', which='major',labelcolor='w')
ax.text(-10,np.max(1e-3*MnSelMvals[:,:,-1]-0.08),'(a)',fontsize=14)
ax  = plt.subplot(312)
for rr,taulevel in enumerate(NameRes):
    ax.plot(nmembsplot,relSdevSelMvals[rr,:,-1],c=ls_colors[rr],linewidth=1.0,marker='o',markersize=4,label=NameLabels[rr])
ax.tick_params(axis='y', which='major', labelsize=14)
ax.set_ylabel(r'$\dfrac{\sigma_{E}}{\sigma_{E,500}}$ in final ice mass',fontsize=14)
ax.tick_params(axis='y', which='major',labelsize=14)
ax.tick_params(axis='x', which='major',labelcolor='w')
ax.text(-10,np.max(relSdevSelMvals[:,:,-1]-0.01),'(b)',fontsize=14)
ax  = plt.subplot(313)
for rr,taulevel in enumerate(NameRes):
    ax.plot(nmembsplot,SkewSelMvals[rr,:,-1],c=ls_colors[rr],linewidth=1.0,marker='o',markersize=4,label=NameLabels[rr])
ax.tick_params(axis='y', which='major', labelsize=14)
ax.set_ylabel('Skew in final ice mass',fontsize=14)
ax.tick_params(axis='y', which='major',labelsize=14)
ax.tick_params(axis='x', which='major',labelsize=14)
ax.text(-10,np.max(SkewSelMvals[:,:,-1]-0.04),'(c)',fontsize=14)
ax.set_xlabel('Number of ensemble members',fontsize=14)
ax.legend(loc='best',fontsize=14,ncol=2)
fig.tight_layout()

